function z = wah(deviation, startfreq, stopfreq, mode)

% define the step size for frequency vector
% adder is the step for arithmetic series (mode = 1)
% multiplr is the step for geometric series (mode = 2)
adder = (10-1)/deviation;
multiplr = 10^(1/deviation);

% determine the exponent k for the highest desired frequency
k = 0;
while 10^k < stopfreq
k = k + 1;
end; % while

% a buffer where the component values 
% are read from a parameter file
RCLvalues = [];
PLOTBUF = [];

% open parameter file, read it line by line,
% and store values to RCLvalues buffer
% by default, the parameter file should be named as params.txt
fid = fopen('params.txt', 'r');
tline = fgetl(fid);
disp(tline);
while 1
tline = fgetl(fid);
if ( ischar(tline) && length(tline) > 1 )
disp(tline);
A = sscanf(tline, '%g');
RCLvalues = [RCLvalues A];
else
break;
endif;
end;
RCLvalues
fclose(fid);

% The outmost loop is determined by 
% the amount of rows in the parameter file.
% This way it is possible to vary component values
% and compare results.
for varied = 1:size(RCLvalues,2)

% read component values into variables
VCC  = RCLvalues(1,varied);
VBE1 = RCLvalues(2,varied);
VBE2 = RCLvalues(3,varied);
R1   = RCLvalues(4,varied);
R2   = RCLvalues(5,varied);
R3   = RCLvalues(6,varied);
R4   = RCLvalues(7,varied);
R5   = RCLvalues(8,varied);
R6   = RCLvalues(9,varied);
R7   = RCLvalues(10,varied);
R81  = RCLvalues(11,varied);
R82  = RCLvalues(12,varied);
R9   = RCLvalues(13,varied);
R10  = RCLvalues(14,varied);
R11  = RCLvalues(15,varied);
RS   = RCLvalues(16,varied);
L1   = RCLvalues(17,varied);
RLL  = RCLvalues(18,varied);
RL   = RLL*R6/(RLL + R6);
C1   = RCLvalues(19,varied);
C2   = RCLvalues(20,varied);
C3   = RCLvalues(21,varied);
C4   = RCLvalues(22,varied);
C5   = RCLvalues(23,varied);

B1   = RCLvalues(24,varied);
B2   = RCLvalues(25,varied);

% determine the DC biasing voltages and currents
OS = VCC + VBE2*(R3/(R11+(B2+1)*R10)) - VBE1 - VBE1*(R3+R5)/R7 - VBE1*(R7+R5)*R3/((R11 + (B2+1)*R10)*R7)
NIM = R5 + RL + R4 + (B1+1)*(R3+R2) + R3*R5/(R11 + (B2+1)*R10)

X1 = (RL + R4 + (B1 + 1)*R2)*(R7+R5)*R3/(R7*(R11 + (B2+1)*R10))
X2 = (RL + R4 + (B1 + 1)*R2)*(R3 + R5)/R7

IBQ1 = OS/(NIM+X1+X2)
IBQ2 = (VCC - IBQ1*(R5 + RL + R4 + (B1 +1)*(R3 + R2)) - ((RL + R4 + (B1 + 1)*R2)/R7)*IBQ1*(R3 + R5) - VBE1*(R3+R5)/R7 - VBE1)/R3 

% determine the transconductance gm and rpi for both transistors 
ICQ1 = B1*IBQ1
gm1 = ICQ1/0.025
rpi1 = B1/gm1

ICQ2 = B2*IBQ2
gm2 = ICQ2/0.025
rpi2 = B2/gm2

V3 = VE1 = (IBQ1 + ICQ1)*R2
V2 = VB1 = VE1 + VBE1
V4 = VC1 = VCC - (ICQ1 + IBQ1 + IBQ2)*R3
V6 = VB2 = VC1 - IBQ2*R11
V7 = VB2 - VBE2
V8 = VC2 = VCC - ICQ2*R9

% define empty arrays for the transfer function H and frequency X
H=[];
X=[];
% start from the frequency of 1 Hz
f = 1;

% loop through all the frequency decades 10, 100, 1000, etc.
% according to the exponent k
for j = 0:k
% loop through the specific decade, e.g. 10, 20, 30, 40, ... 90.
while f <= (10^(j+1) - adder)

if (f >= (startfreq - 0.00001) && f <= stopfreq)

% frequency to angular frequency conversion
w = f*2*pi;

% define the matrix elements from the small-signal model
Y11 = i*w*C1/(1 + i*w*C1*R1) + 1/RS;
Y12 = i*w*C1/(1 + i*w*C1*R1);
Y21 = Y12;
Y22 = i*w*C1/(1 + i*w*C1*R1) + 1/R4 + 1/rpi1;
Y23 = 1/rpi1;
Y29 = 1/R4;
Y32 = Y23;
Y33 = 1/rpi1 + 1/R2;
Y44 = 1/R3 + i*w*C3 + 1/R11 + 1/R5;
Y45 = i*w*C3;
Y46 = 1/R11;
Y410 = 1/R5;
Y54 = Y45;
Y55 = i*w*C3 + 1/R81;
Y511 = 1/R81;
Y64 = Y46;
Y66 = i*w*C4 + 1/R11 + 1/rpi2;
Y67 = 1/rpi2;
Y611 = i*w*C4;
Y76 = Y67;
Y77 = 1/rpi2 + 1/R10 + i*w*C5;
Y79 = i*w*C5;
Y88 = 1/R9;
Y92 = Y29;
Y97 = Y79;
Y99 = 1/R4 + 1/(i*w*L1) + 1/R6 + i*w*C5;
Y910 = 1/R6 + 1/(i*w*L1);
Y104 = Y410;
Y1010 = 1/R5 + 1/R7 + 1/R6 + 1/(i*w*L1) + i*w*C2;
Y109 = Y910;
Y115 = Y511;
Y116 = Y611;
Y1111= 1/R82 + 1/R81 + i*w*C4;

Y32x = -(B1 + 1)/rpi1;
Y33x = (B1 + 1)/rpi1 + 1/R2;
Y42x = B1/rpi1;
Y43x = -Y42x;

Y76x = -(B2 + 1)/rpi2;
Y77x = (B2 + 1)/rpi2 + 1/R10 + i*w*C5;
Y86x = B2/rpi2;
Y87x = -Y86x;

% the numerator matrix (determinant) according to Cramer's rule
% since Y88 is alone in column 8, the matrix can be simplified further

%OSx =[ Y11 -Y12  0    0   1/RS  0    0    0    0    0     0; ...
%      -Y21  Y22 -Y23  0    0    0    0    0   -Y29  0     0; ...
%       0    Y32x Y33x 0    0    0    0    0    0    0     0; ...
%       0    Y42x Y43x Y44  0   -Y46  0    0    0   -Y410  0; ...
%       0    0    0   -Y54  0    0    0    0    0    0    -Y511; ...
%       0    0    0   -Y64  0    Y66 -Y67  0    0    0    -Y611; ...
%       0    0    0    0    0   Y76x  Y77x 0   -Y79  0     0; ...
%       0    0    0    0    0   Y86x  Y87x Y88  0    0     0; ...
%       0   -Y92  0    0    0    0   -Y97  0    Y99 -Y910  0; ...
%       0    0    0   -Y104 0    0    0    0   -Y109 Y1010 0; ...
%       0    0    0    0    0   -Y116 0    0    0    0     Y1111;];

% the denominator matrix (determinant) according to Cramer's rule
% since Y88 is alone in column 8, the matrix can be simplified further

%NIMx = [ Y11 -Y12 0   0    0    0    0    0    0    0     0; ...
%      -Y21  Y22 -Y23  0    0    0    0    0   -Y29  0     0; ...
%       0    Y32x Y33x 0    0    0    0    0    0    0     0; ...
%       0    Y42x Y43x Y44 -Y45 -Y46  0    0    0   -Y410  0; ...
%       0    0    0   -Y54  Y55  0    0    0    0    0    -Y511; ...
%       0    0    0   -Y64  0    Y66 -Y67  0    0    0    -Y611; ...
%       0    0    0    0    0    Y76x Y77x 0   -Y79  0     0; ...
%       0    0    0    0    0    Y86x Y87x Y88  0    0     0; ...
%       0   -Y92  0    0    0    0   -Y97  0    Y99 -Y910  0; ...
%       0    0    0   -Y104 0    0    0    0   -Y109 Y1010 0; ...
%       0    0    0    0   -Y115 -Y116 0    0    0    0     Y1111;];


% the numerator matrix (determinant) according to Cramer's rule
OSx =[ Y11 -Y12  0    0   1/RS  0    0    0    0     0; ...
      -Y21  Y22 -Y23  0    0    0    0   -Y29  0     0; ...
       0    Y32x Y33x 0    0    0    0    0    0     0; ...
       0    Y42x Y43x Y44  0   -Y46  0    0   -Y410  0; ...
       0    0    0   -Y54  0    0    0    0    0    -Y511; ...
       0    0    0   -Y64  0    Y66 -Y67  0    0    -Y611; ...
       0    0    0    0    0   Y76x  Y77x -Y79  0     0; ...
       0   -Y92  0    0    0    0   -Y97  Y99 -Y910  0; ...
       0    0    0   -Y104 0    0    0   -Y109 Y1010 0; ...
       0    0    0    0    0   -Y116 0    0    0     Y1111;];

% the denominator matrix (determinant) according to Cramer's rule
NIMx = [ Y11 -Y12 0   0    0    0    0    0    0     0; ...
      -Y21  Y22 -Y23  0    0    0    0   -Y29  0     0; ...
       0    Y32x Y33x 0    0    0    0    0    0     0; ...
       0    Y42x Y43x Y44 -Y45 -Y46  0    0   -Y410  0; ...
       0    0    0   -Y54  Y55  0    0    0    0    -Y511; ...
       0    0    0   -Y64  0    Y66 -Y67  0    0    -Y611; ...
       0    0    0    0    0    Y76x Y77x -Y79  0     0; ...
       0   -Y92  0    0    0    0   -Y97  Y99 -Y910  0; ...
       0    0    0   -Y104 0    0    0   -Y109 Y1010 0; ...
       0    0    0    0   -Y115 -Y116 0   0    0     Y1111;]; 
 

% store the transfer function values to the H array
% and the frequency values to the X array
H=[H 20*log10(abs(Y88*det(OSx)/(Y88*det(NIMx))))];
X = [X f]; 
endif;

% advance frequency according to the chosen mode
% (arithmetic) or (geometric)
if (mode == 1)
f = f + adder*10^j;
else
f = f*multiplr;
endif;

end; % while
end; % for
PLOTBUF = [PLOTBUF X' H'];
end; % for varied

% save the data for plotting later with Gnuplot
% ngspice generates similar data file, 
% so comparison and plotting against that is simple.
save -ascii wahplot_octa.data PLOTBUF

% handle the Octave specific plotting here
figure (3)
semilogx(X, PLOTBUF(:,2:2:end),'LineWidth',3);
set(gca,'FontSize',14)
set(gca,'YLim',[10*(floor(min(min(PLOTBUF(:,2:2:end)))/10)),10*(ceil(max(max(PLOTBUF(:,2:2:end)))/10))]);
set(gca,'YTick',[10*(floor(min(min(PLOTBUF(:,2:2:end)))/10) - 1):10:10*(ceil(max(max(PLOTBUF(:,2:2:end)))/10) + 1)]);
legend(num2str(PLOTBUF(1,2:2:end)'))
xlabel('frequency [Hz]');
ylabel('magnitude of voltage ratio [dB]');
grid on;
grid minor on;
